unit QzData.Defines;

// QzData.Defines.pas
// Define errors, event handler and constants.

//Copyright (C) Joseph Leung 2005 (author2004 (.at.) quickzip (.dot.) org)
//
//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.
//
//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.
//
//You should have received a copy of the GNU Lesser General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

interface

uses System.Collections, QzHtml.DownloadProxy;

const
  SQLiteDLL = 'sqlite3.dll';
  {$IFDEF CF}
  user32 = 'coredll';
  kernel32 = 'coredll';
  {$ELSE}
  user32 = 'user32.dll';
  kernel32 = 'kernel32.dll';
  {$ENDIF}
  T = 0;
  F = 1;
//  dbdiv = 10000000 ;
type

  rssUpdateEventArgs = class(EventArgs)
    private
    public
      type UpdateType = (utDownload, utParse, utCompleted, utError);
      var work : UpdateType;
      var channel : string;
      var error : string;
      constructor create(aChannel : string; aUpdateType : UpdateType); overload;
      constructor create(aChannel : string; aUpdateType : UpdateType; anError : string); overload;
    end;
  rssUpdateHandler = procedure (Sender:System.Object; e : rssUpdateEventArgs)  of object;

  DBException = Exception;
  sqlCode = record
      id : integer;
      desc : string;
     end;

  ChannelEntry = class
      ChannelRead : boolean;
      ChannelTitle : string;
      ChannelURL : string;
      ChannelWebURL : string;
      ChannelRssVersion : integer;
      ChannelLastUpdate : DateTime;
      ChannelDesc : string;
      ChannelFeedDate : DateTime;
      ChannelCopyRight : string;
      ChannelLanguage : string;
      ChannelMaxFeeds : integer;
      ChannelExtended : integer;
      ChannelLogin : string;
      function ChannelID : string;
      function ChannelPassword : string;
      function getInsertSQL(tableName : string) : string;
      constructor Create(aChannelTitle,aChannelUrl,aChannelWebURL : string; aChannelRssVersion : integer; aChannelLastUpdate : DateTime;
        aChannelDesc : string; aChannelFeedDate : DateTime; aChannelCopyRight, aChannelLanguage : string;
        aChannelMaxFeeds, aChannelExtended : integer; aChannelLogin : string);
      function PrintItem : string;
    end;
  ChannelEntryList = ArrayList;
  FeedEntry = class
      FeedPubDate : DateTime;
      FeedAuthor : string;
      FeedCategory : string;
      FeedLink : string;
      FeedTitle : string;
      FeedDesc : string;
      FeedRead : boolean;
      FeedFav : boolean;
      FeedLock : boolean;
      constructor Create(aFeedAuthor,aFeedCategory,aFeedLink,aFeedTitle,aFeedDesc : string;
        aFeedRead, aFeedFav, aFeedLock  : boolean;
        aFeedPubDate : Datetime);
      function PrintItem : string;
    end;
  FeedEntryList = ArrayList;
{$REGION 'SQLite ReturnValues'}
// Return values for sqlite3_exec() and sqlite3_step()
const
  SQLITE_OK = 0;          // Successful result
  SQLITE_ERROR = 1;       // SQL error or missing database
  SQLITE_INTERNAL = 2;    // An internal logic error in SQLite
  SQLITE_PERM = 3;        // Access permission denied
  SQLITE_ABORT = 4;       // Callback routine requested an abort
  SQLITE_BUSY = 5;        // The database file is locked
  SQLITE_LOCKED = 6;      // A table in the database is locked
  SQLITE_NOMEM = 7;       // A malloc() failed
  SQLITE_READONLY = 8;    // Attempt to write a readonly database
  SQLITE_INTERRUPT = 9;   // Operation terminated by sqlite3_interrupt()
  SQLITE_IOERR = 10;      // Some kind of disk I/O error occurred
  SQLITE_CORRUPT = 11;    // The database disk image is malformed
  SQLITE_NOTFOUND = 12;   // (Internal Only) Table or record not found
  SQLITE_FULL = 13;       // Insertion failed because database is full
  SQLITE_CANTOPEN = 14;   // Unable to open the database file
  SQLITE_PROTOCOL = 15;   // Database lock protocol error
  SQLITE_EMPTY = 16;      // Database is empty
  SQLITE_SCHEMA = 17;     // The database schema changed
  SQLITE_TOOBIG = 18;     // Too much data for one row of a table
  SQLITE_CONSTRAINT = 19; // Abort due to contraint violation
  SQLITE_MISMATCH = 20;   // Data type mismatch
  SQLITE_MISUSE = 21;     // Library used incorrectly
  SQLITE_NOLFS = 22;      // Uses OS features not supported on host
  SQLITE_AUTH = 23;       // Authorization denied
  SQLITE_FORMAT = 24;     // Auxiliary database format error
  SQLITE_RANGE = 25;      // 2nd parameter to sqlite3_bind out of range
  SQLITE_NOTADB = 26;     // File opened that is not a database file
  SQLITE_ROW = 100;       // sqlite3_step() has another row ready
  SQLITE_DONE = 101;      // sqlite3_step() has finished executing
const MaxBuiltinSQLCodes = 29;
const BuiltinSQLCodes: array[1..MaxBuiltinSQLCodes] of sqlCode = (
(id : SQLITE_OK     ; desc : 'Successful result'),
(id : SQLITE_ERROR ; desc : 'SQL error or missing database'),
(id : SQLITE_INTERNAL ; desc : 'An internal logic error in SQLite'),
(id : SQLITE_PERM ; desc : 'Access permission denied'),
(id : SQLITE_ABORT ; desc : 'Callback routine requested an abort'),
(id : SQLITE_BUSY ; desc : 'The database file is locked'),
(id : SQLITE_LOCKED ; desc : 'A table in the database is locked'),
(id : SQLITE_NOMEM ; desc : 'A malloc() failed'),
(id : SQLITE_READONLY ; desc : 'Attempt to write a readonly database'),
(id : SQLITE_INTERRUPT ; desc : 'Operation terminated by sqlite3_interrupt()'),
(id : SQLITE_IOERR ; desc : 'Some kind of disk I/O error occurred'),
(id : SQLITE_CORRUPT ; desc : 'The database disk image is malformed'),
(id : SQLITE_NOTFOUND ; desc : '(Internal Only) Table or record not found'),
(id : SQLITE_FULL ; desc : 'Insertion failed because database is full'),
(id : SQLITE_CANTOPEN ; desc : 'Unable to open the database file'),
(id : SQLITE_PROTOCOL ; desc : 'Database lock protocol error'),
(id : SQLITE_EMPTY ; desc : 'Database is empty'),
(id : SQLITE_SCHEMA ; desc : 'The database schema changed'),
(id : SQLITE_TOOBIG ; desc : 'Too much data for one row of a table'),
(id : SQLITE_CONSTRAINT ; desc : 'Abort due to contraint violation'),
(id : SQLITE_MISMATCH ; desc : 'Data type mismatch'),
(id : SQLITE_MISUSE ; desc : 'Library used incorrectly'),
(id : SQLITE_NOLFS ; desc : 'Uses OS features not supported on host'),
(id : SQLITE_AUTH ; desc : 'Authorization denied'),
(id : SQLITE_FORMAT ; desc : 'Auxiliary database format error'),
(id : SQLITE_RANGE ; desc : '2nd parameter to sqlite3_bind out of range'),
(id : SQLITE_NOTADB ; desc : 'File opened that is not a database file'),
(id : SQLITE_ROW ; desc : 'sqlite3_step() has another row ready'),
(id : SQLITE_DONE ; desc : 'sqlite3_step() has finished executing')
);


{$ENDREGION}
function getDefaultError(code : integer) : string;

implementation

function getDefaultError(code : integer) : string;
var i : integer;
begin
  Result := 'Unknown error code';
  for i := 1 to MaxBuiltinSQLCodes do
    if BuiltinSQLCodes[i].id = code then
      Result := BuiltinSQLCodes[i].desc;
end;

constructor rssUpdateEventArgs.Create(aChannel : string; aUpdateType : UpdateType);
begin
  Create(aChannel,aUpdateType,'');
end;

constructor rssUpdateEventArgs.Create(aChannel : string; aUpdateType : UpdateType; anError : string);
begin
  inherited Create;
  work := aUpdateType;
  Channel := aChannel;
  error := anError;
end;


constructor FeedEntry.Create(aFeedAuthor,aFeedCategory,aFeedLink,aFeedTitle,aFeedDesc : string;
        aFeedRead, aFeedFav, aFeedLock  : boolean;
        aFeedPubDate : Datetime);
begin
  inherited Create;
  FeedPubDate := aFeedPubDate;
  FeedAuthor := aFeedAuthor;
  FeedCategory := aFeedCategory;
  FeedLink := aFeedLink;
  FeedTitle := aFeedTitle;
  FeedDesc := aFeedDesc;
  FeedRead := aFeedRead;
  FeedFav  := aFeedFav;
  Feedlock := aFeedLock;
end;

function FeedEntry.PrintItem : string;
begin
  Result := Convert.tostring(FeedPubDate) + ',' + FeedAuthor + ','+
      {FeedCategory + ',' + FeedLink + ',' + }FeedTitle{ + ',' + FeedDesc};
  Console.WriteLine(Result);
end;

function ChannelEntry.ChannelID : string;
var id, pass : string;
begin
  DecodePassword(ChannelLogin,id,pass);
  Result := id;
end;

function ChannelEntry.ChannelPassword : string;
var id, pass : string;
begin
  DecodePassword(ChannelLogin,id,pass);
  Result := pass;
end;


constructor ChannelEntry.Create(aChannelTitle,aChannelUrl,aChannelWebURL : string; aChannelRssVersion : integer; aChannelLastUpdate : DateTime;
        aChannelDesc : string; aChannelFeedDate : DateTime; aChannelCopyRight, aChannelLanguage : string;
        aChannelMaxFeeds, aChannelExtended : integer; aChannelLogin : string);
begin
  inherited Create;
  ChannelRead := false;
  ChannelTitle := aChannelTitle;
  ChannelUrl := aChannelUrl;
  ChannelWebURL := aChannelWebURL;
  ChannelRssVersion := aChannelRssVersion;
  ChannelLastUpdate := aChannelLastUpdate;
  ChannelDesc := aChannelDesc;
  ChannelFeedDate := aChannelFeedDate;
  ChannelCopyRight := aChannelCopyRight;
  ChannelLanguage := aChannelLanguage;
  ChannelMaxFeeds := aChannelMaxFeeds;
  ChannelExtended := aChannelExtended;
  ChannelLogin := aChannelLogin;
end;

function ChannelEntry.PrintItem : string;
begin
  Result := ChannelTitle + ',' +  ChannelUrl + ',' +
    Convert.tostring(ChannelLastUpdate) + ',' + ChannelCopyRight;
  Console.WriteLine(Result);
end;

function ChannelEntry.getInsertSQL(tableName : string) : string;
begin
  Result := System.String.Format('INSERT INTO "{0}" '+
  '(channel,url,web,rss,lastupdate,desc,feeddate,copyright,lang,maxfeed,ext,login) '+
   'VALUES ("{1}","{2}","{3}","{4}","{5}","{6}","{7}","{8}","{9}",{10},{11},"{12}");',
          [tableName,ChannelTitle,ChannelURL,ChannelWebURL,ChannelRssVersion,Convert.tostring(DateTime.now.Ticks),
           ChannelDesc,Convert.tostring(ChannelFeedDate),ChannelCopyRight,
           ChannelLanguage,ChannelMaxFeeds,ChannelExtended,ChannelLogin ]);
end;


end.
