unit QzData.SqLite;

// QzData.SqLite.pas
// Communicate with Sqlite

//Copyright (C) Joseph Leung 2005 (author2004 (.at.) quickzip (.dot.) org)
//
//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.
//
//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.
//
//You should have received a copy of the GNU Lesser General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


interface
uses QzData.Defines, QzData.TableFormat, System.Collections,
System.Text, System.Runtime.InteropServices, QzComp.Utils, System.IO, System.Reflection ;

type
  QzSqLite = class(System.Object)
    private
      pDBHandle : IntPtr;
      pLastError : string;
    public
      Table : dbTable;
      property LastError : string read pLastError;

      constructor Create(dbname : string); override;
      destructor Destroy; override;

      function checkError(returnValue : integer; expectedValues : Array of Integer) : boolean;

      function ExecSQL(SQL : string) : integer;
      procedure BeginTransaction;
      procedure Commit;

      function OpenTable(sql : string; maxFeed : integer) : boolean; overload;
      function OpenTable(sql : string) : boolean; overload;
      function RowCount(sql : string) : integer;

      function TableExists(sql : string) : boolean;
      function CreateTable(tablename, additionalSQL : string; structure : dbColHeaderList) : boolean;
      function DropTable(tablename : string) : boolean;
      procedure AddRow(tablename : string; row : dbRowEntry);
      procedure RemoveRow(tablename : string; row : dbRowEntry; const basedOnKeys : Array of const);
      procedure UpdateRow(tablename : string; row : dbRowEntry; const basedOnKeys : Array of const;
          const Keys2Change : Array of const);
      procedure Compact;
    end;

implementation
{$REGION 'DllHeader'}

[DllImport(SQLiteDLL)] function  sqlite3_open(dbname: IntPtr; var db: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_close(db: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_exec(db: IntPtr; SQLStatement: IntPtr; CallbackPtr: IntPtr; Sender: TObject; var ErrMsg: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_version(): IntPtr; external;
[DllImport(SQLiteDLL)] function  sqlite3_errmsg(db: IntPtr): IntPtr; external;
[DllImport(SQLiteDLL)] function  sqlite3_errcode(db: IntPtr): integer; external;
[DllImport(SQLiteDLL)] procedure sqlite3_free(P: IntPtr); external;
[DllImport(SQLiteDLL)] function  sqlite3_complete(P: IntPtr): boolean; external;
[DllImport(SQLiteDLL)] function  sqlite3_last_insert_rowid(db: IntPtr): int64; external;
[DllImport(SQLiteDLL)] procedure sqlite3_interrupt(db: IntPtr); external;
[DllImport(SQLiteDLL)] procedure sqlite3_busy_timeout(db: IntPtr; TimeOut: integer); external;
[DllImport(SQLiteDLL)] function  sqlite3_changes(db: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_total_changes(db: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_prepare(db: IntPtr; SQLStatement: IntPtr; nBytes: integer; var hStmt: IntPtr; var pzTail: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_count(hStmt: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_name(hStmt: IntPtr; ColNum: integer): IntPtr; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_decltype(hStmt: IntPtr; ColNum: integer): IntPtr; external;
[DllImport(SQLiteDLL)] function  sqlite3_step(hStmt: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_data_count(hStmt: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_blob(hStmt: IntPtr; ColNum: integer): IntPtr; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_bytes(hStmt: IntPtr; ColNum: integer): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_double(hStmt: IntPtr; ColNum: integer): double; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_int(hStmt: IntPtr; ColNum: integer): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_text(hStmt: IntPtr; ColNum: integer): IntPtr; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_type(hStmt: IntPtr; ColNum: integer): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_column_int64(hStmt: IntPtr; ColNum: integer): Int64; external;
[DllImport(SQLiteDLL)] function  sqlite3_finalize(hStmt: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_reset(hStmt: IntPtr): integer; external;
[DllImport(SQLiteDLL)] function  sqlite3_findblob(hStmt: IntPtr; ParamNum: integer;
  ptrData: IntPtr; numBytes: integer; ptrDestructor: IntPtr): integer; external;
//[DllImport(SQLiteDLL)] function  sqlitefieldtype(SQLiteFieldTypeCode: Integer): AnsiString; external;
//[DllImport(SQLiteDLL)] function  sqliteerrorstr(SQLiteErrorCode: Integer): AnsiString; external;

[DllImport(kernel32)]  function GlobalAlloc(uFlags : LongWord;  uBytes : Integer) : IntPtr; external;
{$ENDREGION}

constructor QzSqLite.Create(dbname : string);
begin
  inherited Create;
  if not checkError(
    sqlite3_Open(IntPtrStr.Create(dbName).asPointer,pDbHandle),
      [SQLITE_OK,SQLITE_DONE]) then
    raise DBException.Create('Unable to open ' + dbName);
    ExecSQL('PRAGMA SYNCHRONOUS=NORMAL;');
    ExecSQL('PRAGMA temp_store = MEMORY;');
end;

destructor QzSqLite.Destroy;
begin
  if Assigned(pDBHandle) then
    begin
      SQLite3_Close(pDBHandle);
      pDBHandle := IntPtr.Zero;
    end;
  inherited Destroy;
end;


function QzSqLite.ExecSQL(SQL : string) : integer;
var Stmt: IntPtr;
    NextSQLStatement: IntPtr;
begin
  Result := -1;
  if Assigned(pDBHandle) then
  try
    Result := sqlite3_Prepare(pDBHandle, IntPtrStr.Create(SQL).asPointer, -1, Stmt, NextSQLStatement);
    if (not checkError(Result,[SQLITE_OK,SQLITE_DONE])) or (Stmt = nil) then
          raise DBException.Create('Could not prepare SQL statement ' + SQL + #13#10 + LastError) else
            Result := sqlite3_step(Stmt);
  finally
    if Assigned(Stmt) then
      sqlite3_Finalize(stmt);
  end;
end;

function QzSqLite.checkError(returnValue : integer; expectedValues : Array of Integer) : boolean;
var Msg : IntPtr;
  function matchValue : boolean;
  var i : integer;
  begin
    Result := false;
    for i := 0 to length(expectedValues) -1 do
      if expectedValues[i] = returnValue then
        begin
          Result := true;
          exit;
        end;
  end;
begin
  if matchValue then Result := true
  else begin
          Result := false;
          pLastError := getDefaultError(returnValue);
          if Assigned(pDBHandle) then
            try
              Msg := sqlite3_ErrMsg(pDBHandle);
              if Msg <> IntPtr.Zero then
                pLastError := IntPtr2Str(Msg,Encoding.ASCII)
            finally
              if Assigned(Msg) then sqlite3_Free(Msg);
              Console.WriteLine(pLastError);
            end;
       end;
end;

function QzSqLite.CreateTable(tablename, additionalSQL : string; structure : dbColHeaderList) : boolean;
var sql : string;
begin
  Result := false;
  if TableExists('SELECT * FROM ' + tablename) then Result := true else
    begin
      BeginTransaction;
      sql := System.&String.Format(structure.getCreateSQL,[tablename,additionalSQL]);
      ExecSql(sql);
      Commit;
      if TableExists('SELECT * FROM ' + tablename) then Result := true;
    end;
end;

function QzSqLite.DropTable(tablename : string) : boolean;
begin
  Result := false;
  if not TableExists('SELECT * FROM ' + tablename) then Result := true else
    begin
      BeginTransaction;
      ExecSql('DROP TABLE ' + tablename);
      Commit;
      if not TableExists('SELECT * FROM ' + tablename) then Result := true;
    end;  
end;

procedure QzSqLite.BeginTransaction;
begin
  ExecSQL('BEGIN TRANSACTION;');
end;

procedure QzSqLite.Commit;
begin
  ExecSQL('COMMIT;');
end;

function QzSqLite.TableExists(sql : string) : boolean;
var lResult : integer;
    Stmt: IntPtr;
    NextSQLStatement: IntPtr;
begin
  Result := false;
  try
    Result :=
      (sqlite3_Prepare(pDBHandle,
        IntPtrStr.Create(SQL).asPointer, -1, Stmt, NextSQLStatement) = SQLITE_OK);
  finally
    if Assigned(Stmt) then
      sqlite3_Finalize(stmt);
  end;
end;

procedure QzSqLite.AddRow(tablename : string; row : dbRowEntry);
var s, sql : string;
begin
  if OpenTable('SELECT * FROM "' + tablename + '"',0) then
    begin
      sql := System.&String.Format(
        row.getInsertSQL(Table.CollumnHeader),
          [tablename]);
      BeginTransaction;
      ExecSQL(sql);
      Commit;
    end;
end;

procedure QzSqLite.RemoveRow(tablename : string; row : dbRowEntry; const basedOnKeys : Array of const);
var sql : string;
begin
  sql := System.&String.Format(
        row.getDeleteSQL(basedOnKeys),
          [tablename]);
  BeginTransaction;
  ExecSQL(sql);
  Commit;
end;

procedure QzSqLite.UpdateRow(tablename : string; row : dbRowEntry; const basedOnKeys : Array of const;
          const Keys2Change : Array of const);
var sql : string;
begin
  sql := System.&String.Format(
        row.getUpdateSQL(Table.CollumnHeader,basedOnKeys,Keys2Change),
          [tablename]);
  BeginTransaction;
  ExecSQL(sql);
  Commit;
end;

function QzSqLite.RowCount(sql : string) : integer;
var lResult : integer;
    Stmt: IntPtr;
  function Prepare(sql : string) : integer;
  var NextSQLStatement: IntPtr;
  begin
    Result := sqlite3_Prepare(pDBHandle, IntPtrStr.Create(SQL).asPointer, -1, Stmt, NextSQLStatement);
  end;
begin
  Result := 0;
  Table := dbTable.Create;
  try
    lResult := Prepare(sql);

    Case lResult of
    SQLITE_OK,SQLITE_ROW :
      begin
//        Inc(Result);
        while (Sqlite3_step(Stmt) = SQLITE_ROW) do Inc(Result);
      end;
    SQLITE_ERROR :
      begin
        checkError(lResult,[]);
        raise Exception.Create('Database not exists');
      end;

    else raise Exception.Create('Error when opening table');
    end;
  finally
    if Assigned(Stmt) then
      sqlite3_Finalize(stmt);
  end;
end;

function QzSqLite.OpenTable(sql : string) : boolean;
begin
  Result := OpenTable(sql,65535);
end;

function QzSqLite.OpenTable(sql : string; maxFeed : integer) : boolean;
var lResult : integer;
    Stmt: IntPtr;
  function Prepare(sql : string) : integer;
  var NextSQLStatement: IntPtr;
  begin
    Result := sqlite3_Prepare(pDBHandle, IntPtrStr.Create(SQL).asPointer, -1, Stmt, NextSQLStatement);
  end;
  function sType2dbType(cType : string) : dbType;
  begin
    Result := dtUnknown;
     if (ctype.tolower.IndexOf('integer') <> -1) then Result := dtInt else
     if (ctype.tolower.IndexOf('int') <> -1)     then Result := dtInt else
     if (ctype.tolower.IndexOf('numeric') <> -1) then Result := dtFloat else
     if (ctype.tolower.IndexOf('float') <> -1)   then Result := dtFloat else
     if (ctype.tolower.IndexOf('double') <> -1)  then Result := dtFloat else
     if (ctype.tolower.IndexOf('real') <> -1)    then Result := dtFloat else
     if (ctype.tolower.IndexOf('blob') <> -1)    then Raise Exception.Create('Blob not supported') else
      Result := dtString;
  end;
  function ReadCollumn : boolean;
  var ColCount : integer;
      i : integer;
      colType,colName : string;
      cdbType : dbType;
  begin
    result := false;
    ColCount := Sqlite3_Column_Count(Stmt);
    for i := 0 to ColCount -1 do
      begin
        colName := IntPtrStr.Create(Sqlite3_Column_Name(stmt, i)).AsString;
        colType := IntPtrStr.Create(Sqlite3_Column_DeclType(stmt, i)).AsString;
        cdbType := sType2dbType(colType);

        if cdbType = dtString then
          if (ColName.tolower.IndexOf('date') <> -1) or
             (ColName.tolower.IndexOf('time') <> -1) or
             (ColName.ToLower = 'feeddate') or
             (ColName.ToLower = 'lastupdate') then
              cdbType := dtDateTime;
          if (ColName.ToLower = 'read') or
             (ColName.ToLower = 'fav') or
             (ColName.ToLower = 'lock') then
              cdbType := dtBool;

        //TODO : Determine dbtype dynamically.
        Table.AddCol(colName,cdbType);
      end;
    result := ColCount > 0;
  end;
  procedure ReadRow;
  var i : integer;
      row : dbRowEntry;
      Key : string;
      dt : Datetime;
      value : Int64;
  begin
    row := dbRowEntry.Create;
    for i := 0 to Table.CollumnHeader.Count -1 do
        begin
          Key := Table.CollumnHeader.Item[i].key;
          Case Table.CollumnHeader.Item[i].value of
          dtString    : row.AddString(Key,IntPtrStr.Create(Sqlite3_Column_Text(stmt, i)).asString);
          dtDateTime  :
            begin
              try
                value := Convert.ToInt64(IntPtrStr.Create(Sqlite3_Column_Text(stmt, i)).asString);
                dt := Datetime.Create(value);
              except
                Console.WriteLine(value.ToString +' is not valid datetime value');
                dt := Datetime.Now;
              end;
              row.AddDateTime(Key,dt);
            end;
          dtInt       : row.AddInt(Key,sqlite3_column_int(stmt,i));
          dtBool      : row.AddBool(Key,(sqlite3_column_int(stmt,i)=T));
          dtFloat     : row.AddFloat(Key,sqlite3_column_double(stmt,i));
          else raise Exception.Create('Unknown Collumn type');
          end;
        end;
    Table.AddRow(row);
  end;

begin
  Result := false;
  Table := dbTable.Create;
  try
    lResult := Prepare(sql);

    Case lResult of
    SQLITE_OK,SQLITE_ROW : if ReadCollumn and (maxFeed > 0) then
                 begin
                  while (Sqlite3_step(Stmt) = SQLITE_ROW) and
                    (Table.Count < maxFeed) do
                    ReadRow;
                 end;
    SQLITE_ERROR :
      begin
        checkError(lResult,[]);
        raise Exception.Create('Database not exists');
      end;

    else raise Exception.Create('Error when opening table');
    end;
    Result := true;
  finally
    if Assigned(Stmt) then
      sqlite3_Finalize(stmt);
  end;
end;

procedure QzSqLite.Compact;
begin
//  sqlite3_exec(pDBHandle,IntPtrStr.Create('VACUUM').asPointer,0,0);
end;


end.
